---- MODULE MemoirCommon ----

(***************************************************************************)
(* This module defines some basic constants used by both the high-level    *)
(* and low-level specs.                                                    *)
(*                                                                         *)
(* A developer that wishes to use `^\name^' is expected to provide a       *)
(* service implementation that (1) operates on some application-specific   *)
(* input, (2) produces some application-specific output, and (3) maintains *)
(* some application-specific public and private state.  The developer also *)
(* specifies an initial public and private state for the service.  The     *)
(* service is assumed to be type-safe.                                     *)
(*                                                                         *)
(* The one non-obvious aspect of this module is the constant               *)
(* InitialAvailableInputs, which will be explained in the comments         *)
(* relating to the high-level spec.                                        *)
(*                                                                         *)
(***************************************************************************)

EXTENDS TLAPS

CONSTANT InputType \*`^\label{CONSTANT-InputType}^'
CONSTANT OutputType \*`^\label{CONSTANT-OutputType}^'
CONSTANT PublicStateType \*`^\label{CONSTANT-PublicStateType}^'
CONSTANT PrivateStateType \*`^\label{CONSTANT-PrivateStateType}^'

ServiceResultType == \*`^\label{DEF-ServiceResultType}^'
    [   newPublicState : PublicStateType,
        newPrivateState : PrivateStateType,
        output : OutputType
    ]

CONSTANT Service(_,_,_) \*`^\label{CONSTANT-Service}^'

ASSUME ServiceTypeSafe == \*`^\label{ASSUME-ServiceTypeSafe}^'
    \A input \in InputType, publicState \in PublicStateType, privateState \in PrivateStateType :
        Service(publicState, privateState, input) \in ServiceResultType

CONSTANT InitialAvailableInputs \*`^\label{CONSTANT-InitialAvailableInputs}^'
CONSTANT InitialPublicState \*`^\label{CONSTANT-InitialPublicState}^'
CONSTANT InitialPrivateState \*`^\label{CONSTANT-InitialPrivateState}^'
CONSTANT DeadPublicState \*`^\label{CONSTANT-DeadPublicState}^'
CONSTANT DeadPrivateState \*`^\label{CONSTANT-DeadPrivateState}^'

ASSUME ConstantsTypeSafe == \*`^\label{ASSUME-ConstantsTypeSafe}^'
    /\ InitialAvailableInputs \subseteq InputType
    /\ InitialPublicState \in PublicStateType
    /\ InitialPrivateState \in PrivateStateType
    /\ DeadPublicState \in PublicStateType
    /\ DeadPrivateState \in PrivateStateType

====
