---- MODULE MemoirLL2ImplementationLemmas ----

(***************************************************************************)
(* This module states and proves a bunch of lemmas that support the proof  *)
(* that the `^\fastname^' spec implements the `^\basicname^' spec.         *)
(*                                                                         *)
(* This module includes the following theorems:                            *)
(*     UnchangedAvailableInputsLemma                                       *)
(*     UnchangedObservedOutputsLemma                                       *)
(*     UnchangedObservedAuthenticatorsLemma                                *)
(*     UnchangedDiskPublicStateLemma                                       *)
(*     UnchangedDiskPrivateStateEncLemma                                   *)
(*     UnchangedDiskHistorySummaryLemma                                    *)
(*     UnchangedDiskAuthenticatorLemma                                     *)
(*     UnchangedDiskLemma                                                  *)
(*     UnchangedRAMPublicStateLemma                                        *)
(*     UnchangedRAMPrivateStateEncLemma                                    *)
(*     UnchangedRAMHistorySummaryLemma                                     *)
(*     UnchangedRAMAuthenticatorLemma                                      *)
(*     UnchangedRAMLemma                                                   *)
(*     UnchangedNVRAMHistorySummaryLemma                                   *)
(*     UnchangedNVRAMSymmetricKeyLemma                                     *)
(*     UnchangedNVRAMLemma                                                 *)
(***************************************************************************)

EXTENDS MemoirLL2RefinementLemmas

(***************************************************************************)
(* The UnchangedAvailableInputsLemma states that when there is no change   *)
(* to the `^\fastname^' variable representing the available itputs, there  *)
(* is no change to the `^\basicname^' variable representing the available  *)
(* itputs.                                                                 *)
(***************************************************************************)

THEOREM UnchangedAvailableInputsLemma == \*`^\label{THEOREM-UnchangedAvailableInputsLemma}^'
      ( /\  UNCHANGED LL2AvailableInputs
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1AvailableInputs
  <1>1. HAVE /\  UNCHANGED LL2AvailableInputs
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL2AvailableInputs
    BY <1>1
  <1>3. LL1AvailableInputs = LL2AvailableInputs
    BY <1>1 DEF LL2Refinement
  <1>4. LL1AvailableInputs' = LL2AvailableInputs'
    BY <1>1 DEF LL2Refinement
  <1>5. QED
    BY <1>2, <1>3, <1>4

----

(***************************************************************************)
(* The UnchangedObservedOutputsLemma states that when there is no change   *)
(* to the `^\fastname^' variable representing the observed outputs, there  *)
(* is no change to the `^\basicname^' variable representing the observed   *)
(* outputs.                                                                *)
(***************************************************************************)

THEOREM UnchangedObservedOutputsLemma == \*`^\label{THEOREM-UnchangedObservedOutputsLemma}^'
      ( /\  UNCHANGED LL2ObservedOutputs
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1ObservedOutputs
  <1>1. HAVE /\  UNCHANGED LL2ObservedOutputs
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL2ObservedOutputs
    BY <1>1
  <1>3. LL1ObservedOutputs = LL2ObservedOutputs
    BY <1>1 DEF LL2Refinement
  <1>4. LL1ObservedOutputs' = LL2ObservedOutputs'
    BY <1>1 DEF LL2Refinement
  <1>5. QED
    BY <1>2, <1>3, <1>4

----

(***************************************************************************)
(* The UnchangedObservedAuthenticatorsLemma states that when there is no   *)
(* change to the `^\fastname^' variable representing the observed          *)
(* authenticators, there is no change to the `^\basicname^' variable       *)
(* representing the observed authenticators.                               *)
(***************************************************************************)

THEOREM UnchangedObservedAuthenticatorsLemma == \*`^\label{THEOREM-UnchangedObservedAuthenticatorsLemma}^'
      ( /\  UNCHANGED LL2ObservedAuthenticators
        /\  UNCHANGED LL2NVRAM.symmetricKey
        /\  UNCHANGED LL2NVRAM.hashBarrier
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1ObservedAuthenticators
  <1>1. HAVE /\  UNCHANGED LL2ObservedAuthenticators
             /\  UNCHANGED LL2NVRAM.symmetricKey
             /\  UNCHANGED LL2NVRAM.hashBarrier
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL2ObservedAuthenticators
    BY <1>1
  <1>3. UNCHANGED LL2NVRAM.symmetricKey
    BY <1>1
  <1>4. UNCHANGED LL2NVRAM.hashBarrier
    BY <1>1
  <1>5. AuthenticatorSetsMatch(
            LL1ObservedAuthenticators,
            LL2ObservedAuthenticators,
            LL2NVRAM.symmetricKey,
            LL2NVRAM.hashBarrier)
    BY <1>1 DEF LL2Refinement
  <1>6. AuthenticatorSetsMatch(
            LL1ObservedAuthenticators',
            LL2ObservedAuthenticators',
            LL2NVRAM.symmetricKey',
            LL2NVRAM.hashBarrier')
    BY <1>1 DEF LL2Refinement
  <1>7. QED
    <2>1. LL1ObservedAuthenticators \in SUBSET MACType
      BY <1>1 DEF LL2Refinement
    <2>2. LL1ObservedAuthenticators' \in SUBSET MACType
      BY <1>1 DEF LL2Refinement
    <2>3. LL2ObservedAuthenticators \in SUBSET MACType
      BY <1>1 DEF LL2TypeInvariant
    <2>4. LL2NVRAM.symmetricKey \in SymmetricKeyType
      BY <1>1, LL2SubtypeImplicationLemma DEF LL2SubtypeImplication
    <2>5. LL2NVRAM.hashBarrier \in HashType
      BY <1>1, LL2SubtypeImplicationLemma DEF LL2SubtypeImplication
    <2>6. QED
      BY <1>2, <1>3, <1>4, <1>5, <1>6, <2>1, <2>2, <2>3, <2>4, <2>5,
          AuthenticatorSetsMatchUniqueLemma

----

(***************************************************************************)
(* The UnchangedDiskPublicStateLemma states that when there is no change   *)
(* to the field of the `^\fastname^' variable representing the public      *)
(* state in the disk, there is no change to the field of the               *)
(* `^\basicname^' variable representing the public state in the disk.      *)
(***************************************************************************)

THEOREM UnchangedDiskPublicStateLemma == \*`^\label{THEOREM-UnchangedDiskPublicStateLemma}^'
      ( /\  UNCHANGED LL2Disk.publicState
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1Disk.publicState
  <1>1. HAVE /\  UNCHANGED LL2Disk.publicState
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL2Disk.publicState
    BY <1>1
  <1>3. LL1Disk.publicState = LL2Disk.publicState
    BY <1>1 DEF LL2Refinement
  <1>4. LL1Disk.publicState' = LL2Disk.publicState'
    BY <1>1 DEF LL2Refinement
  <1>5. QED
    BY <1>2, <1>3, <1>4

----

(***************************************************************************)
(* The UnchangedDiskPrivateStateEncLemma states that when there is no      *)
(* change to the field of the `^\fastname^' variable representing the      *)
(* encrypted private state in the disk, there is no change to the field of *)
(* the `^\basicname^' variable representing the encrypted private state in *)
(* the disk.                                                               *)
(***************************************************************************)

THEOREM UnchangedDiskPrivateStateEncLemma == \*`^\label{THEOREM-UnchangedDiskPrivateStateEncLemma}^'
      ( /\  UNCHANGED LL2Disk.privateStateEnc
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1Disk.privateStateEnc
  <1>1. HAVE /\  UNCHANGED LL2Disk.privateStateEnc
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL2Disk.privateStateEnc
    BY <1>1
  <1>3. LL1Disk.privateStateEnc = LL2Disk.privateStateEnc
    BY <1>1 DEF LL2Refinement
  <1>4. LL1Disk.privateStateEnc' = LL2Disk.privateStateEnc'
    BY <1>1 DEF LL2Refinement
  <1>5. QED
    BY <1>2, <1>3, <1>4

----

(***************************************************************************)
(* The UnchangedDiskHistorySummaryLemma states that when there is no       *)
(* change to the field of the `^\fastname^' variable representing the      *)
(* history summary in the disk, there is no change to the field of the     *)
(* `^\basicname^' variable representing the history summary in the disk.   *)
(***************************************************************************)

THEOREM UnchangedDiskHistorySummaryLemma == \*`^\label{THEOREM-UnchangedDiskHistorySummaryLemma}^'
      ( /\  UNCHANGED LL2Disk.historySummary
        /\  UNCHANGED LL2NVRAM.hashBarrier
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1Disk.historySummary
  <1>1. HAVE /\  UNCHANGED LL2Disk.historySummary
             /\  UNCHANGED LL2NVRAM.hashBarrier
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL2Disk.historySummary
    BY <1>1
  <1>3. UNCHANGED LL2NVRAM.hashBarrier
    BY <1>1
  <1>4. HistorySummariesMatch(
            LL1Disk.historySummary,
            LL2Disk.historySummary,
            LL2NVRAM.hashBarrier)
    BY <1>1 DEF LL2Refinement
  <1>5. HistorySummariesMatch(
            LL1Disk.historySummary',
            LL2Disk.historySummary',
            LL2NVRAM.hashBarrier')
    BY <1>1 DEF LL2Refinement
  <1>6. QED
    <2>1. LL1Disk.historySummary \in HashType
      BY <1>1 DEF LL2Refinement, LL1UntrustedStorageType
    <2>2. LL1Disk.historySummary' \in HashType
      BY <1>1 DEF LL2Refinement, LL1UntrustedStorageType
    <2>3. LL2Disk.historySummary \in HistorySummaryType
      BY <1>1, LL2SubtypeImplicationLemma DEF LL2SubtypeImplication
    <2>4. LL2NVRAM.hashBarrier \in HashType
      BY <1>1, LL2SubtypeImplicationLemma DEF LL2SubtypeImplication
    <2>5. QED
      BY <1>2, <1>3, <1>4, <1>5, <2>1, <2>2, <2>3, <2>4,
      HistorySummariesMatchUniqueLemma

----

(***************************************************************************)
(* The UnchangedDiskAuthenticatorLemma states that when there is no change *)
(* to the field of the `^\fastname^' variable representing the             *)
(* authenticator in the disk, there is no change to the field of the       *)
(* `^\basicname^' variable representing the authenticator in the disk.     *)
(***************************************************************************)

THEOREM UnchangedDiskAuthenticatorLemma == \*`^\label{THEOREM-UnchangedDiskAuthenticatorLemma}^'
      ( /\  UNCHANGED LL2Disk.authenticator
        /\  UNCHANGED LL2NVRAM.hashBarrier
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1Disk.authenticator
  <1>1. HAVE /\  UNCHANGED LL2Disk.authenticator
             /\  UNCHANGED LL2NVRAM.hashBarrier
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL2Disk.authenticator
    BY <1>1
  <1>3. UNCHANGED LL2NVRAM.hashBarrier
    BY <1>1
  <1>4. \E symmetricKey \in SymmetricKeyType :
            AuthenticatorsMatch(
                LL1Disk.authenticator,
                LL2Disk.authenticator,
                symmetricKey,
                LL2NVRAM.hashBarrier)
    BY <1>1 DEF LL2Refinement
  <1>5. \E symmetricKey \in SymmetricKeyType :
            AuthenticatorsMatch(
                LL1Disk.authenticator',
                LL2Disk.authenticator',
                symmetricKey,
                LL2NVRAM.hashBarrier')
    BY <1>1 DEF LL2Refinement
  <1>6. QED
    <2>1. LL1Disk.authenticator \in MACType
      BY <1>1 DEF LL2Refinement, LL1UntrustedStorageType
    <2>2. LL1Disk.authenticator' \in MACType
      BY <1>1 DEF LL2Refinement, LL1UntrustedStorageType
    <2>3. LL2Disk.authenticator \in MACType
      BY <1>1, LL2SubtypeImplicationLemma DEF LL2SubtypeImplication
    <2>4. LL2NVRAM.hashBarrier \in HashType
      BY <1>1, LL2SubtypeImplicationLemma DEF LL2SubtypeImplication
    <2>5. QED
      BY <1>2, <1>3, <1>4, <1>5, <2>1, <2>2, <2>3, <2>4,
          AuthenticatorsMatchUniqueLemma

----

(***************************************************************************)
(* The UnchangedDiskLemma states that when there is no change to the       *)
(* `^\fastname^' variable representing the disk, there is no change to the *)
(* `^\basicname^' variable representing the disk.                          *)
(***************************************************************************)

THEOREM UnchangedDiskLemma == \*`^\label{THEOREM-UnchangedDiskLemma}^'
      ( /\  UNCHANGED LL2Disk
        /\  UNCHANGED LL2NVRAM.symmetricKey
        /\  UNCHANGED LL2NVRAM.hashBarrier
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1Disk
  <1>1. HAVE /\  UNCHANGED LL2Disk
             /\  UNCHANGED LL2NVRAM.symmetricKey
             /\  UNCHANGED LL2NVRAM.hashBarrier
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL1Disk.publicState
    <2>1. UNCHANGED LL2Disk.publicState
      BY <1>1
    <2>2. QED
      BY <1>1, <2>1, UnchangedDiskPublicStateLemma
  <1>3. UNCHANGED LL1Disk.privateStateEnc
    <2>1. UNCHANGED LL2Disk.privateStateEnc
      BY <1>1
    <2>2. QED
      BY <1>1, <2>1, UnchangedDiskPrivateStateEncLemma
  <1>4. UNCHANGED LL1Disk.historySummary
    <2>1. UNCHANGED LL2Disk.historySummary
      BY <1>1
    <2>2. QED
      BY <1>1, <2>1, UnchangedDiskHistorySummaryLemma
  <1>5. UNCHANGED LL1Disk.authenticator
    <2>1. UNCHANGED LL2Disk.authenticator
      BY <1>1
    <2>2. QED
      BY <1>1, <2>1, UnchangedDiskAuthenticatorLemma
  <1>6. LL1Disk \in LL1UntrustedStorageType
    BY <1>1 DEF LL2Refinement
  <1>7. LL1Disk' \in LL1UntrustedStorageType
    BY <1>1 DEF LL2Refinement
  <1>8. QED
    BY <1>2, <1>3, <1>4, <1>5, <1>6, <1>7, LL1DiskRecordCompositionLemma

----

(***************************************************************************)
(* The UnchangedRAMPublicStateLemma states that when there is no change to *)
(* the field of the `^\fastname^' variable representing the public state   *)
(* in the RAM, there is no change to the field of the `^\basicname^'       *)
(* variable representing the public state in the RAM.                      *)
(***************************************************************************)

THEOREM UnchangedRAMPublicStateLemma == \*`^\label{THEOREM-UnchangedRAMPublicStateLemma}^'
      ( /\  UNCHANGED LL2RAM.publicState
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1RAM.publicState
  <1>1. HAVE /\  UNCHANGED LL2RAM.publicState
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL2RAM.publicState
    BY <1>1
  <1>3. LL1RAM.publicState = LL2RAM.publicState
    BY <1>1 DEF LL2Refinement
  <1>4. LL1RAM.publicState' = LL2RAM.publicState'
    BY <1>1 DEF LL2Refinement
  <1>5. QED
    BY <1>2, <1>3, <1>4

----

(***************************************************************************)
(* The UnchangedRAMPrivateStateEncLemma states that when there is no       *)
(* change to the field of the `^\fastname^' variable representing the      *)
(* encrypted private state in the RAM, there is no change to the field of  *)
(* the `^\basicname^' variable representing the encrypted private state in *)
(* the RAM.                                                                *)
(***************************************************************************)

THEOREM UnchangedRAMPrivateStateEncLemma == \*`^\label{THEOREM-UnchangedRAMPrivateStateEncLemma}^'
      ( /\  UNCHANGED LL2RAM.privateStateEnc
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1RAM.privateStateEnc
  <1>1. HAVE /\  UNCHANGED LL2RAM.privateStateEnc
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL2RAM.privateStateEnc
    BY <1>1
  <1>3. LL1RAM.privateStateEnc = LL2RAM.privateStateEnc
    BY <1>1 DEF LL2Refinement
  <1>4. LL1RAM.privateStateEnc' = LL2RAM.privateStateEnc'
    BY <1>1 DEF LL2Refinement
  <1>5. QED
    BY <1>2, <1>3, <1>4

----

(***************************************************************************)
(* The UnchangedRAMHistorySummaryLemma states that when there is no change *)
(* to the field of the `^\fastname^' variable representing the history     *)
(* summary in the RAM, there is no change to the field of the              *)
(* `^\basicname^' variable representing the history summary in the RAM.    *)
(***************************************************************************)

THEOREM UnchangedRAMHistorySummaryLemma == \*`^\label{THEOREM-UnchangedRAMHistorySummaryLemma}^'
      ( /\  UNCHANGED LL2RAM.historySummary
        /\  UNCHANGED LL2NVRAM.hashBarrier
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1RAM.historySummary
  <1>1. HAVE /\  UNCHANGED LL2RAM.historySummary
             /\  UNCHANGED LL2NVRAM.hashBarrier
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL2RAM.historySummary
    BY <1>1
  <1>3. UNCHANGED LL2NVRAM.hashBarrier
    BY <1>1
  <1>4. HistorySummariesMatch(
            LL1RAM.historySummary,
            LL2RAM.historySummary,
            LL2NVRAM.hashBarrier)
    BY <1>1 DEF LL2Refinement
  <1>5. HistorySummariesMatch(
            LL1RAM.historySummary',
            LL2RAM.historySummary',
            LL2NVRAM.hashBarrier')
    BY <1>1 DEF LL2Refinement
  <1>6. QED
    <2>1. LL1RAM.historySummary \in HashType
      BY <1>1 DEF LL2Refinement, LL1UntrustedStorageType
    <2>2. LL1RAM.historySummary' \in HashType
      BY <1>1 DEF LL2Refinement, LL1UntrustedStorageType
    <2>3. LL2RAM.historySummary \in HistorySummaryType
      BY <1>1, LL2SubtypeImplicationLemma DEF LL2SubtypeImplication
    <2>4. LL2NVRAM.hashBarrier \in HashType
      BY <1>1, LL2SubtypeImplicationLemma DEF LL2SubtypeImplication
    <2>5. QED
      BY <1>2, <1>3, <1>4, <1>5, <2>1, <2>2, <2>3, <2>4,
      HistorySummariesMatchUniqueLemma

----

(***************************************************************************)
(* The UnchangedRAMAuthenticatorLemma states that when there is no change  *)
(* to the field of the `^\fastname^' variable representing the             *)
(* authenticator in the RAM, there is no change to the field of the        *)
(* `^\basicname^' variable representing the authenticator in the RAM.      *)
(***************************************************************************)

THEOREM UnchangedRAMAuthenticatorLemma == \*`^\label{THEOREM-UnchangedRAMAuthenticatorLemma}^'
      ( /\  UNCHANGED LL2RAM.authenticator
        /\  UNCHANGED LL2NVRAM.hashBarrier
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1RAM.authenticator
  <1>1. HAVE /\  UNCHANGED LL2RAM.authenticator
             /\  UNCHANGED LL2NVRAM.hashBarrier
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL2RAM.authenticator
    BY <1>1
  <1>3. UNCHANGED LL2NVRAM.hashBarrier
    BY <1>1
  <1>4. \E symmetricKey \in SymmetricKeyType :
            AuthenticatorsMatch(
                LL1RAM.authenticator,
                LL2RAM.authenticator,
                symmetricKey,
                LL2NVRAM.hashBarrier)
    BY <1>1 DEF LL2Refinement
  <1>5. \E symmetricKey \in SymmetricKeyType :
            AuthenticatorsMatch(
                LL1RAM.authenticator',
                LL2RAM.authenticator',
                symmetricKey,
                LL2NVRAM.hashBarrier')
    BY <1>1 DEF LL2Refinement
  <1>6. QED
    <2>1. LL1RAM.authenticator \in MACType
      BY <1>1 DEF LL2Refinement, LL1UntrustedStorageType
    <2>2. LL1RAM.authenticator' \in MACType
      BY <1>1 DEF LL2Refinement, LL1UntrustedStorageType
    <2>3. LL2RAM.authenticator \in MACType
      BY <1>1, LL2SubtypeImplicationLemma DEF LL2SubtypeImplication
    <2>4. LL2NVRAM.hashBarrier \in HashType
      BY <1>1, LL2SubtypeImplicationLemma DEF LL2SubtypeImplication
    <2>5. QED
      BY <1>2, <1>3, <1>4, <1>5, <2>1, <2>2, <2>3, <2>4,
          AuthenticatorsMatchUniqueLemma

----

(***************************************************************************)
(* The UnchangedRAMLemma states that when there is no change to the        *)
(* `^\fastname^' variable representing the RAM, there is no change to the  *)
(* `^\basicname^' variable representing the RAM.                           *)
(***************************************************************************)

THEOREM UnchangedRAMLemma == \*`^\label{THEOREM-UnchangedRAMLemma}^'
      ( /\  UNCHANGED LL2RAM
        /\  UNCHANGED LL2NVRAM.symmetricKey
        /\  UNCHANGED LL2NVRAM.hashBarrier
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1RAM
  <1>1. HAVE /\  UNCHANGED LL2RAM
             /\  UNCHANGED LL2NVRAM.symmetricKey
             /\  UNCHANGED LL2NVRAM.hashBarrier
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL1RAM.publicState
    <2>1. UNCHANGED LL2RAM.publicState
      BY <1>1
    <2>2. QED
      BY <1>1, <2>1, UnchangedRAMPublicStateLemma
  <1>3. UNCHANGED LL1RAM.privateStateEnc
    <2>1. UNCHANGED LL2RAM.privateStateEnc
      BY <1>1
    <2>2. QED
      BY <1>1, <2>1, UnchangedRAMPrivateStateEncLemma
  <1>4. UNCHANGED LL1RAM.historySummary
    <2>1. UNCHANGED LL2RAM.historySummary
      BY <1>1
    <2>2. QED
      BY <1>1, <2>1, UnchangedRAMHistorySummaryLemma
  <1>5. UNCHANGED LL1RAM.authenticator
    <2>1. UNCHANGED LL2RAM.authenticator
      BY <1>1
    <2>2. QED
      BY <1>1, <2>1, UnchangedRAMAuthenticatorLemma
  <1>6. LL1RAM \in LL1UntrustedStorageType
    BY <1>1 DEF LL2Refinement
  <1>7. LL1RAM' \in LL1UntrustedStorageType
    BY <1>1 DEF LL2Refinement
  <1>8. QED
    BY <1>2, <1>3, <1>4, <1>5, <1>6, <1>7, LL1RAMRecordCompositionLemma

----

(***************************************************************************)
(* The UnchangedNVRAMHistorySummaryLemma states that when there is no      *)
(* change to the LL2NVRAMLogicalHistorySummary value, which represents the *)
(* logical value of the history summary in the NVRAM and SPCR of the       *)
(* `^\fastname^' spec, there is no change to the field of the              *)
(* `^\basicname^' variable representing the history summary in the NVRAM.  *)
(***************************************************************************)

THEOREM UnchangedNVRAMHistorySummaryLemma == \*`^\label{THEOREM-UnchangedNVRAMHistorySummaryLemma}^'
      ( /\  UNCHANGED LL2NVRAMLogicalHistorySummary
        /\  UNCHANGED LL2NVRAM.symmetricKey
        /\  UNCHANGED LL2NVRAM.hashBarrier
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1NVRAM.historySummary
  <1>1. HAVE /\  UNCHANGED LL2NVRAMLogicalHistorySummary
             /\  UNCHANGED LL2NVRAM.symmetricKey
             /\  UNCHANGED LL2NVRAM.hashBarrier
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL2NVRAMLogicalHistorySummary
    BY <1>1
  <1>3. UNCHANGED LL2NVRAM.symmetricKey
    BY <1>1
  <1>4. UNCHANGED LL2NVRAM.hashBarrier
    BY <1>1
  <1>5. HistorySummariesMatch(
            LL1NVRAM.historySummary,
            LL2NVRAMLogicalHistorySummary,
            LL2NVRAM.hashBarrier)
    BY <1>1 DEF LL2Refinement
  <1>6. HistorySummariesMatch(
            LL1NVRAM.historySummary',
            LL2NVRAMLogicalHistorySummary',
            LL2NVRAM.hashBarrier')
    BY <1>1 DEF LL2Refinement
  <1>7. QED
    <2>1. LL1NVRAM.historySummary \in HashType
      BY <1>1 DEF LL2Refinement, LL1TrustedStorageType
    <2>2. LL1NVRAM.historySummary' \in HashType
      BY <1>1 DEF LL2Refinement, LL1TrustedStorageType
    <2>3. LL2NVRAMLogicalHistorySummary\in HistorySummaryType
      BY <1>1, LL2NVRAMLogicalHistorySummaryTypeSafe
    <2>4. LL2NVRAM.hashBarrier \in HashType
      BY <1>1, LL2SubtypeImplicationLemma DEF LL2SubtypeImplication
    <2>5. QED
      BY <1>2, <1>3, <1>4, <1>5, <1>6, <2>1, <2>2, <2>3, <2>4,
      HistorySummariesMatchUniqueLemma

----

(***************************************************************************)
(* The UnchangedNVRAMSymmetricKeyLemma states that when there is no change *)
(* to the field of the `^\fastname^' variable representing the symmetric   *)
(* key in the RAM, there is no change to the field of the `^\basicname^'   *)
(* variable representing the symmetric key in the RAM.                     *)
(***************************************************************************)

THEOREM UnchangedNVRAMSymmetricKeyLemma == \*`^\label{THEOREM-UnchangedNVRAMSymmetricKeyLemma}^'
      ( /\  UNCHANGED LL2NVRAM.symmetricKey
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1NVRAM.symmetricKey
  <1>1. HAVE /\  UNCHANGED LL2NVRAM.symmetricKey
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL2NVRAM.symmetricKey
    BY <1>1
  <1>3. LL1NVRAM.symmetricKey = LL2NVRAM.symmetricKey
    BY <1>1 DEF LL2Refinement
  <1>4. LL1NVRAM.symmetricKey' = LL2NVRAM.symmetricKey'
    BY <1>1 DEF LL2Refinement
  <1>5. QED
    BY <1>2, <1>3, <1>4

----

(***************************************************************************)
(* The UnchangedNVRAMLemma states that when there is no change to the      *)
(* `^\fastname^' variables representing the NVRAM and SPCR, there is no    *)
(* change to the `^\basicname^' variable representing the NVRAM.           *)
(***************************************************************************)

THEOREM UnchangedNVRAMLemma == \*`^\label{THEOREM-UnchangedNVRAMLemma}^'
      ( /\  UNCHANGED LL2NVRAM
        /\  UNCHANGED LL2SPCR
        /\  LL2Refinement
        /\  LL2Refinement'
        /\  LL2TypeInvariant
        /\  LL2TypeInvariant' )
    =>
        UNCHANGED LL1NVRAM
  <1>1. HAVE /\  UNCHANGED LL2NVRAM
             /\  UNCHANGED LL2SPCR
             /\  LL2Refinement
             /\  LL2Refinement'
             /\  LL2TypeInvariant
             /\  LL2TypeInvariant'
  <1>2. UNCHANGED LL1NVRAM.historySummary
    <2>1. UNCHANGED LL2NVRAMLogicalHistorySummary
      BY <1>1 DEF LL2NVRAMLogicalHistorySummary
    <2>2. QED
      BY <1>1, <2>1, UnchangedNVRAMHistorySummaryLemma
  <1>3. UNCHANGED LL1NVRAM.symmetricKey
    <2>1. UNCHANGED LL2NVRAM.symmetricKey
      BY <1>1
    <2>2. QED
      BY <1>1, <2>1, UnchangedNVRAMSymmetricKeyLemma
  <1>4. LL1NVRAM \in LL1TrustedStorageType
    BY <1>1 DEF LL2Refinement
  <1>5. LL1NVRAM' \in LL1TrustedStorageType
    BY <1>1 DEF LL2Refinement
  <1>6. QED
    BY <1>2, <1>3, <1>4, <1>5, LL1NVRAMRecordCompositionLemma

====
